#!/usr/bin/env perl -T
###############################################################################
#
#    Copyright (C) 2002-2004 by Eric Gerbier
#    Bug reports to: eric.gerbier@tutanota.com
#    $Id: Cfg.pm 1572 2009-04-02 12:27:27Z gerbier $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this classes implement the afick report function
# could be a set of function, but maybe add also data
#
# this is a bad but temporary depends : this code functions from
# main afick code : report_summary/display_changes/ctimef/parent_date
###############################################################################
# I use special naming for references :
# $r_name : for a ref to scalar
# $rh_name : for a ref to hashes
# $ra_name : for a ref to array

package Afick::Report;

use strict;
use warnings;
use File::Basename;    # dirname

#use threads;
#use threads::shared qw(share shared_clone);

use Afick::Msg;
use Afick::Constant;
use Afick::Package;

our ($VERSION);
$VERSION = '1.5';

use English qw(-no_match_vars);

###############################################################
# constuctor
sub new($) {
	my $classe = shift @_;

	my $self = {};

	#share($self);
	$self = bless $self, $classe;

	$self->_init();
	return $self;
}

###############################################################
# configure initial data
# ref https://www.perlmonks.org/?node_id=259551
sub _init($) {
	my $self = shift @_;

	# finded : keep a record of scanned files
	my %h1;
	$self->{'finded'} = \%h1;

	# dangling : list of dangling files
	my %h2;
	$self->{'dangling'} = \%h2;

	# newval
	# keep new record of changes files
	my %h3;
	$self->{'newval'} = \%h3;

	# oldval
	# keep previous record of changes files
	my %h4;
	$self->{'oldval'} = \%h4;

	# newfiles
	# new files count
	my %h5;
	$self->{'newfiles'} = \%h5;

	# masq
	# new files count
	my %h6;
	$self->{'masq'} = \%h6;

	# update
	# update
	$self->{'update'} = 0;

	# nb_degraded
	$self->{'nb_degraded'} = 0;

	# count system changes
	$self->{'masked_sysupdate'} = 0;

	# Afick::Package
	$self->{'package'} = Afick::Package->new();
	return;
}
#######################################################
# update mode (used in wanted_update)
sub set_update($$) {
	my $self = shift @_;
	my $val  = shift @_;

	$self->{'update'} = $val;
	return;
}

#------------------------------------------------------
sub is_update($) {
	my $self = shift @_;

	return $self->{'update'};
}
#######################################################
sub nb_masked_sysupdate($) {
	my $self = shift @_;

	return $self->{'masked_sysupdate'};
}
#######################################################
sub get_nb_degraded($) {
	my $self = shift @_;

	return $self->{'nb_degraded'};
}

#------------------------------------------------------
sub inc_nb_degraded($) {
	my $self = shift @_;

	return $self->{'nb_degraded'}++;
}
#######################################################
# finded
#######################################################
sub set_finded($$) {
	my $self = shift @_;
	my $name = shift @_;

	$self->{'finded'}->{$name} = 1;

	return;
}

#------------------------------------------------------
sub is_finded($$) {
	my $self = shift @_;
	my $name = shift @_;

	return exists $self->{'finded'}->{$name};
}

#------------------------------------------------------
sub nb_finded($) {
	my $self = shift @_;

	return scalar keys %{ $self->{'finded'} };
}
#######################################################
# dangling
#######################################################
sub set_dangling($$$) {
	my $self  = shift @_;
	my $name  = shift @_;
	my $value = shift @_;

	$self->{'dangling'}->{$name} = $value;

	return;
}

#------------------------------------------------------
sub get_dangling($$$) {
	my $self = shift @_;
	my $name = shift @_;

	return $self->{'dangling'}->{$name};
}

#------------------------------------------------------
sub keys_dangling($) {
	my $self = shift @_;

	return keys %{ $self->{'dangling'} };
}

#------------------------------------------------------
sub nb_dangling($) {
	my $self = shift @_;

	return scalar keys %{ $self->{'dangling'} };
}

#------------------------------------------------------
# print all danglink symbolic links
sub print_dangling($$$) {
	my $self     = shift @_;
	my $detailed = shift @_;    # true or false
	my $config   = shift @_;    # Afick::Cfg

	Afick::Msg->debug( "(print_dangling) begin detailed=$detailed", 1 );
	if ( $config->get_directive('warn_dead_symlinks') ) {

		# warnings
		foreach my $key ( $self->keys_dangling() ) {
			main::report_summary( 'Dangling', $key );
			$self->report_detailed( 'linked_to', $self->get_dangling($key) )
			  if ($detailed);
		}
	}
	elsif ($detailed) {

		# just info
		foreach my $key ( $self->keys_dangling() ) {
			Afick::Msg->debug( "Dangling file : $key", 1 );
		}
	}
	Afick::Msg->debug( '(print_dangling) end', 1 );
	return;
}
#######################################################
# print run conditions
sub print_env($$$$$$) {
	my $self         = shift @_;
	my $action       = shift @_;    # ex init, update ...
	my $config       = shift @_;    # Afick::Cfg
	my $dbm          = shift @_;    # database type
	my $date_ref_hum = shift @_;    # date of run
	my $version      = shift @_;    # afick version

	Afick::Msg->info(
			"Afick ($version) $action at $date_ref_hum with options ("
		  . $config->get_configfile()
		  . '):' );

	# only show set directives
	$config->print_directives(0);

	# and dbm as pseudo-directive
	Afick::Msg->info( 'dbm' . Afick::Constant->DIREC . $dbm );
	return;
}
#######################################################
# print last run date and version
sub print_last($$$) {
	my $self        = shift @_;
	my $run_date    = shift @_;    # last scan date
	my $old_version = shift @_;    # afick old version

	if ( defined $run_date ) {
		Afick::Msg->info( 'last run on '
			  . $run_date
			  . ' with afick version '
			  . $old_version );
	}
	return;
}
#######################################################
# high-level sub for detailed messages
sub report_detailed($$$;$) {
	my $self  = shift @_;
	my $field = shift @_;    # field name
	my $val1  = shift @_;    # old value
	my $val2  = shift @_;    # new value (optionnal)

	my $txt = "\t$field\t\t : $val1";
	$txt .= "\t$val2" if ( defined $val2 );

	Afick::Msg->report( $txt, Afick::Msg->Info );
	return $txt;
}
#######################################################
# masq
#######################################################
sub set_masq($$$) {
	my $self  = shift @_;
	my $name  = shift @_;
	my $value = shift @_;

	$self->{'masq'}->{$name} = $value;

	return;
}

#------------------------------------------------------
sub get_masq($$$) {
	my $self = shift @_;
	my $name = shift @_;

	return $self->{'masq'}->{$name};
}
#######################################################
# oldval
#######################################################
sub set_oldval($$$) {
	my $self  = shift @_;
	my $name  = shift @_;
	my $value = shift @_;

	$self->{'oldval'}->{$name} = $value;

	return;
}

#------------------------------------------------------
sub get_oldval($$$) {
	my $self = shift @_;
	my $name = shift @_;

	return $self->{'oldval'}->{$name};
}

#------------------------------------------------------
sub keys_oldval($) {
	my $self = shift @_;

	return keys %{ $self->{'oldval'} };
}

#------------------------------------------------------
sub nb_oldval($) {
	my $self = shift @_;

	return scalar keys %{ $self->{'oldval'} };
}
#######################################################
# newval
#######################################################
sub set_newval($$$) {
	my $self  = shift @_;
	my $name  = shift @_;
	my $value = shift @_;

	$self->{'newval'}->{$name} = $value;

	return;
}

#------------------------------------------------------
sub get_newval($$$) {
	my $self = shift @_;
	my $name = shift @_;

	return $self->{'newval'}->{$name};
}

#------------------------------------------------------
sub keys_newval($) {
	my $self = shift @_;

	return keys %{ $self->{'newval'} };
}

#------------------------------------------------------
sub nb_newval($) {
	my $self = shift @_;

	return scalar keys %{ $self->{'newval'} };
}

#------------------------------------------------------
sub is_newval($$) {
	my $self = shift @_;
	my $name = shift @_;

	return exists $self->{'newval'}->{$name};
}

#------------------------------------------------------
# print a summary or detailed info on changes
sub print_changed($$$) {
	my $self     = shift @_;
	my $detailed = shift @_;    # true or false
	my $config   = shift @_;

	foreach my $key ( sort $self->keys_oldval ) {
		if (    ( $config->get_directive('mask_sysupdate') )
			and ( $self->{'package'}->is_dbconform($key) ) )
		{
			$self->{'masked_sysupdate'}++;
			Afick::Msg->debug( "(print_changed) skip sysupdate $key", 2 );
			delete $self->{'oldval'}->{$key};
		}
		else {
			main::report_summary( 'changed', $key );
			my $old = $self->get_oldval($key);
			$old->display_changes( $self->get_newval($key), $config, $self )
			  if ($detailed);
		}
	}
	return;
}
#######################################################
# newfiles
#######################################################
sub set_newfiles($$$) {
	my $self  = shift @_;
	my $name  = shift @_;    # filename
	my $value = shift @_;    # level

	$self->{'newfiles'}->{$name} = $value;

	return;
}

#------------------------------------------------------
sub get_newfiles($$$) {
	my $self = shift @_;
	my $name = shift @_;

	return $self->{'newfiles'}->{$name};
}

#------------------------------------------------------
sub keys_newfiles($) {
	my $self = shift @_;

	return keys %{ $self->{'newfiles'} };
}

#------------------------------------------------------
sub nb_newfiles($) {
	my $self = shift @_;

	return scalar keys %{ $self->{'newfiles'} };
}

#------------------------------------------------------
sub is_newfiles($$) {
	my $self = shift @_;
	my $name = shift @_;

	return exists $self->{'newfiles'}->{$name};
}
#######################################################
# display as detailed info the inode last change date
sub print_new($$$) {
	my $self     = shift @_;
	my $detailed = shift @_;    # true or false
	my $config   = shift @_;    # Afick::Cfg

	my %news;
	my $nb     = 0;             # partial number
	my $nbnew  = 0;             # total number
	my $topdir = q{};

	# look for new files
	foreach my $key ( sort $self->keys_newfiles() ) {
		Afick::Msg->debug( "(print_new) $key " . $self->get_newfiles($key), 4 );

		# get directory for a file or parent directory for a directory
		my $path = dirname($key);

		# report only if asked all ($Report_full_newdel) or first level
		if (    ( $config->get_directive('mask_sysupdate') )
			and ( $self->{'package'}->is_dbconform($key) ) )
		{
			if ($nb) {

				# display previous partial sum
				#$self->report_detailed( "number of new files", $nb );
				$self->report_detailed( "$topdir number of new files", $nb );
			}
			$nb = 0;

			$self->{'masked_sysupdate'}++;
			Afick::Msg->debug( "(print_new) skip sysupdate $key", 2 );
		}
		elsif ( $config->get_directive('report_full_newdel') ) {
			main::report_summary( 'new', $key );
			$self->report_detailed( 'inode_date', main::ctimef($key) )
			  if ($detailed);
		}
		elsif ( exists $news{$path} ) {
			$news{$key} = $news{$path} + 1;
			$nb++;
		}
		else {
			$news{$key} = 1;
			if ($nb) {

				# display previous partial sum
				#$self->report_detailed( "$topdir number of new files", $nb );
				$self->report_detailed( "number of new files", $nb );
			}
			$nb     = 0;
			$topdir = $key;
			main::report_summary( 'new', $key );
			$self->report_detailed( 'inode_date', main::ctimef($key) )
			  if ($detailed);
		}
		$nbnew++;

	}    # foreach
	if ($nb) {
		#$self->report_detailed( "$topdir number of new files", $nb );
		$self->report_detailed( "number of new files", $nb );
	}
	return $nbnew;
}
#######################################################
# display as info the date change of parent directory
sub print_delete($$$$) {
	my $self     = shift @_;
	my $detailed = shift @_;    # true or false
	my $config   = shift @_;    # Afick::Cfg
	my $backend  = shift @_;    # Afick::Backend

	my %deleted;
	my $nb       = 0;
	my $nbdelete = 0;
	my $topdir   = q{};
	foreach my $key ( sort $backend->getkeys() ) {

		#Afick::Msg->info("print_delete backend $key");
		if ( !$self->is_finded($key) ) {

# in mask_sysupdate we would mask deleted files from removed packages
# but there is no easy way to check if a removed file belong the a package or not
# with local rpm or dpkg command ( only with apt/dnf with a network call )
#			if (    ( $config->get_directive('mask_sysupdate') )
#				and ( !$self->{'package'}->file2pac($key) ) )
#			{
#				$self->{'masked_sysupdate'}++;
#				Afick::Msg->debug( "(print_delete) skip sysupdate $key", 2 );
#			}

			#Afick::Msg->info("print_delete ! is_finded $key");

			# get directory for a file or parent directory for a directory
			my $path = dirname($key);

			#print "--- deleted $key $path";

			if ( $config->get_directive('report_full_newdel') ) {
				if ($nb) {

					# display previous partial sum
					$self->report_detailed( 'number of deleted files', $nb );
				}
				$nb = 0;

				main::report_summary( 'deleted', $key );
				$self->report_detailed( 'parent_date',
					main::parent_date($path) )
				  if ($detailed);
			}
			elsif ( exists $deleted{$path} ) {
				$deleted{$key} = $deleted{$path} + 1;
				$nb++;
			}
			else {
				$deleted{$key} = 1;
				if ($nb) {

					# display previous partial sum
					$self->report_detailed( 'number of deleted files', $nb );
				}
				$nb     = 0;
				$topdir = $key;
				main::report_summary( 'deleted', $key );
				$self->report_detailed( 'parent_date',
					main::parent_date($path) )
				  if ($detailed);
			}

			$nbdelete++;

			# do not delete data in summary mode : we need then in detailed mode
			$backend->del($key) if ( $detailed and $backend->is_update() );
		}    # is_finded
	}    # foreach
	if ($nb) {
		$self->report_detailed( 'number of deleted files', $nb );
	}
	return $nbdelete;
}
#######################################################
1;
