#!/usr/bin/env perl -T
###############################################################################
#
#    Copyright (C) 2002-2004 by Eric Gerbier
#    Bug reports to: eric.gerbier@tutanota.com
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a classe for thread needs
###############################################################################
# I use special naming for references :
# $r_name : for a ref to scalar
# $rh_name : for a ref to hashes
# $ra_name : for a ref to array

package Afick::Thread;

use strict;
use warnings;

use English qw(-no_match_vars);
use Config;    # is_threads_available

use threads;
use threads::shared;
use Thread::Queue;

use Exporter;
use base qw(Exporter);

## no critic (ProhibitAutomaticExportation)
our ( $VERSION, @EXPORT, );

$VERSION = 0.2;
@EXPORT =
  qw( &is_threads_available &get_nb_thread &thread_init &thread_end &thread_submit &thread_on );

use Afick::Msg;
use Afick::Tst;    # is_linux
use Afick::Gen;    # get_nb_proc

my $Nb_threads;    # number of threads ( >= 0)
my $Th_queue;      # threads queue
my @Stats : shared;

##########################################################################
# return true if threads are available
sub is_threads_available() {

	# we will use Config (cf perlthrtut)
	return $Config{'useithreads'};
}
##########################################################################
# load perl threads modules
#sub threads_init_modules() {
#	my @list = ( 'threads', 'threads::shared', 'Thread::Queue' );
#	main::load_modules(@list);
#	return;
#}
##########################################################################
# convert directive in number of threads
# if < 0 : no threads
# if = 0 : number of proc
# if > 0 : set number
sub get_nb_thread($) {
	my $config = shift @_;

	if ( !defined $Nb_threads ) {

		# no cache, compute
		Afick::Msg->debug(" thread config : $config");
		if ( $config < 0 ) {
			$Nb_threads = 0;
			Afick::Msg->debug('do not use threads');
		}
		elsif ( is_threads_available() ) {
			$Nb_threads = ($config) ? $config : get_nb_proc();
			Afick::Msg->info("use $Nb_threads threads");
		}
		else {
			Afick::Msg->warning('no threads available');
			$Nb_threads = 0;
		}
	}
	return $Nb_threads;
}
##########################################################################
# thread main code : get data from queue and call wanted_update
sub thread_core($) {
	my $flot = shift @_;

	my $thr = threads->self;
	my $tid = $thr->tid();
	Afick::Msg->debug("begin of thread $tid");

	while ( my $item = $flot->dequeue() ) {
		@Stats[$tid]++;

		# call appropriate sub
		my $key  = $item->{'key'};
		my $masq = $item->{'masq'};

		$thr->yield();

		#Afick::Msg->info("$tid treat $key $masq");
		main::wanted_update( $key, $masq );

		$thr->yield();
	}

	Afick::Msg->debug("end of thread $tid");
	return;
}
##########################################################################
# send data to queue
sub thread_submit($$) {
	my $key  = shift @_;
	my $masq = shift @_;

	if ( thread_on() ) {
		my %h = (
			'key'  => $key,
			'masq' => $masq,
		);

		#Afick::Msg->info("submit $key $masq");
		$Th_queue->enqueue( \%h );
	}
	else {
		main::wanted_update( $key, $masq );
	}
	return;
}
##########################################################################
# return true if threads are used
sub thread_on() {
	return $Nb_threads > 0;
}
##########################################################################
# create threads to execute thread_core code
sub thread_init($) {
	my $threads_config = shift @_;

	my $nb_th = get_nb_thread($threads_config);
	if ( $nb_th > 0 ) {
		$Th_queue = Thread::Queue->new;
		foreach my $i ( 1 .. $nb_th ) {
			my $th = threads->new( \&thread_core, $Th_queue );
			if ($th) {
				Afick::Msg->debug("creation threads $i");
			}else {
				Afick::Msg->warning("problem creation threads $i");
			}
		}
	}
	return;
}
##########################################################################
# wait until all threads are finished
sub thread_end() {

	if ( thread_on() ) {
		Afick::Msg->debug('end queue');

		# send end queue
		$Th_queue->end();

		foreach my $thr ( threads->list() ) {
			my $tid = $thr->tid;
			Afick::Msg->debug("wait join $tid");
			$thr->join;
			Afick::Msg->debug("join $tid ok");
		}
		Afick::Msg->debug('join ok');
		$Nb_threads = 0;

		# stats for each thread
		my $i = 0;
		foreach my $val (@Stats) {
			$val = (defined $val) ? $val : 0;
			Afick::Msg->info("stat thread $i : $val");
			$i++;
		}
	}
	return;
}
##########################################################################
1;
